@@ -1,5 +1,5 @@
 import warnings
-from typing import Any, Callable, Dict, List, Set, Type
+from typing import Any, Callable, Dict, List, Optional, Set, Type
 
 from docutils import nodes
 from docutils.nodes import Element, Node
@@ -42,7 +42,7 @@ class DummyOptionSpec(dict):
 class DocumenterBridge:
     """A parameters container for Documenters."""
 
-    def __init__(self, env: BuildEnvironment, reporter: Reporter, options: Options,
+    def __init__(self, env: BuildEnvironment, reporter: Optional[Reporter], options: Options,
                  lineno: int, state: Any) -> None:
         self.env = env
         self._reporter = reporter
